@@ -1,5 +1,5 @@
 # Archive.pm - This module does the archiving job for a mailing lists.
-# RCS Identication ; $Revision: 1.16 $ ; $Date: 2006/02/13 14:29:44 $ 
+# RCS Identication ; $Revision: 4025 $ ; $Date: 2006-11-21 12:01:03 +0100 (mar, 21 nov 2006) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -158,13 +158,13 @@ sub scan_dir_archive {
 	my $msg = {};
 	$msg->{'id'} = $i;
 
-	$msg->{'subject'}  = &MIME::Words::decode_mimewords($mail->{'msg'}->head->get('Subject'));
+	$msg->{'subject'} = &MIME::EncWords::decode_mimewords($mail->{'msg'}->head->get('Subject'), Charset=>'utf8');
 	chomp $msg->{'subject'};
 
-	$msg->{'from'}= &MIME::Words::decode_mimewords($mail->{'msg'}->head->get('From'));
+	$msg->{'from'} = &MIME::EncWords::decode_mimewords($mail->{'msg'}->head->get('From'), Charset=>'utf8');
 	chomp $msg->{'from'};    	        	
         
-	$msg->{'date'} = $mail->{'msg'}->head->get('Date');
+	$msg->{'date'} = &MIME::EncWords::decode_mimewords($mail->{'msg'}->head->get('Date'), Charset=>'utf8');
 	chomp $msg->{'date'};
 	
 	$msg->{'full_msg'} = $mail->{'msg'}->as_string;
