@@ -1,5 +1,5 @@
 # Conf.pm - This module does the sympa.conf and robot.conf parsing
-# RCS Identication ; $Revision: 1.138.2.2 $ ; $Date: 2006/08/24 15:05:36 $ 
+# RCS Identication ; $Revision: 4576 $ ; $Date: 2007-08-28 14:21:40 +0200 (mar, 28 aoû 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -32,22 +32,29 @@ require Exporter;
 use Carp;
 
 @ISA = qw(Exporter);
-@EXPORT = qw(%Conf);
+@EXPORT = qw(%Conf DAEMON_MESSAGE DAEMON_COMMAND DAEMON_CREATION DAEMON_ALL);
+
+require 'tools.pl';
+
+sub DAEMON_MESSAGE {1};
+sub DAEMON_COMMAND {2};
+sub DAEMON_CREATION {4};
+sub DAEMON_ALL {7};
 
 my @valid_options = qw(
 		       avg bounce_warn_rate bounce_halt_rate bounce_email_prefix chk_cert_expiration_task expire_bounce_task
 		       cache_list_config
-		       clean_delay_queue clean_delay_queueauth clean_delay_queuemod clean_delay_queuesubscribe clean_delay_queuetopic default_remind_task
-		       cookie cookie_cas_expire create_list crl_dir crl_update_task db_host db_env db_name db_timeout
+		       clean_delay_queue clean_delay_queueauth clean_delay_queuemod clean_delay_queuesubscribe clean_delay_queueautomatic clean_delay_queuetopic default_remind_task
+		       cookie cookie_cas_expire create_list automatic_list_feature automatic_list_creation automatic_list_removal crl_dir crl_update_task db_host db_env db_name db_timeout
 		       db_options db_passwd db_type db_user db_port db_additional_subscriber_fields db_additional_user_fields
 		       default_shared_quota default_archive_quota default_list_priority distribution_mode edit_list email etc
 		       global_remind home host domain lang listmaster listmaster_email localedir log_socket_type log_level 
 		       logo_html_definition misaddressed_commands misaddressed_commands_regexp max_size maxsmtp nrcpt 
-		       owner_priority pidfile pidfile_distribute
-		       spool queue queuedistribute queueauth queuetask queuebounce queuedigest 
-		       queuemod queuetopic queuesubscribe queueoutgoing tmpdir
+		       owner_priority pidfile pidfile_distribute pidfile_creation
+		       spool queue queuedistribute queueauth queuetask queuebounce queuedigest queueautomatic
+		       queuemod queuetopic queuesubscribe queueoutgoing tmpdir lock_method
 		       loop_command_max loop_command_sampling_delay loop_command_decrease_factor loop_prevention_regex
-		       purge_user_table_task  purge_orphan_bounces_task eval_bouncers_task process_bouncers_task
+		       purge_user_table_task purge_logs_table_task purge_orphan_bounces_task eval_bouncers_task process_bouncers_task
 		       minimum_bouncing_count minimum_bouncing_period bounce_delay 
 		       default_bounce_level1_rate default_bounce_level2_rate 
 		       remind_return_path request_priority return_path_suffix rfc2369_header_fields sendmail sendmail_args sleep 
@@ -56,16 +63,22 @@ my @valid_options = qw(
 		       antivirus_path antivirus_args antivirus_notify anonymous_header_fields sendmail_aliases
 		       dark_color light_color text_color bg_color error_color selected_color shaded_color
 		       color_0 color_1 color_2 color_3 color_4 color_5 color_6 color_7 color_8 color_9 color_10 color_11 color_12 color_13 color_14 color_15
- 		       css_url css_path
+ 		       css_url css_path static_content_path static_content_url pictures_max_size pictures_feature
 		       ldap_export_name ldap_export_host ldap_export_suffix ldap_export_password
 		       ldap_export_dnmanager ldap_export_connection_timeout update_db_field_types urlize_min_size
-		       list_check_smtp list_check_suffixes  spam_protection web_archive_spam_protection soap_url
-		       web_recode_to
+		       list_check_smtp list_check_suffixes filesystem_encoding spam_protection web_archive_spam_protection soap_url
+		       use_blacklist
 );
 
 my %old_options = ('trusted_ca_options' => 'capath,cafile',
 		   'msgcat' => 'localedir',
-		   'queueexpire' => '');
+		   'queueexpire' => '',
+		   'web_recode_to' => 'filesystem_encoding',
+		   );
+## These parameters now have a hard-coded value
+## Customized value can be accessed though as %Ignored_Conf
+%Ignored_Conf;
+my %hardcoded_options = ('filesystem_encoding' => 'utf8');
 
 my %valid_options = ();
 map { $valid_options{$_}++; } @valid_options;
@@ -76,7 +89,7 @@ my %Default_Conf =
      'key_passwd' => '',
      'ssl_cert_dir' => '--EXPL_DIR--/X509-user-certs',
      'crl_dir' => '--EXPL_DIR--/crl',
-     'umask'   => '027',
+     'umask'   => '022',
      'syslog'  => 'LOCAL1',
      'log_level'  => 0,
      'nrcpt'   => 25,
@@ -91,11 +104,13 @@ my %Default_Conf =
      'email'   => 'sympa',
      'pidfile' => '--PIDDIR--/sympa.pid',
      'pidfile_distribute' => '--PIDDIR--/sympa-distribute.pid',
+     'pidfile_creation' => '--PIDDIR--/sympa-creation.pid',
      'localedir'  => '--LOCALEDIR--',
      'sort'    => 'fr,ca,be,ch,uk,edu,*,com',
      'spool'   => '--SPOOLDIR--',
      'queue'   => undef,
      'queuedistribute' => undef,
+     'queueautomatic' => undef,
      'queuedigest'=> undef,
      'queuemod'   => undef,
      'queuetopic' => undef,
@@ -110,6 +125,7 @@ my %Default_Conf =
      'clean_delay_queuemod' => 10,
      'clean_delay_queuetopic' => 7,
      'clean_delay_queuesubscribe' => 10,
+     'clean_delay_queueautomatic' => 10,
      'clean_delay_queueauth' => 3,
      'log_socket_type'      => 'unix',
      'log_smtp'      => '',
@@ -139,6 +155,9 @@ my %Default_Conf =
      'max_size' => 5242880,
      'edit_list' => 'owner',
      'create_list' => 'public_listmaster',
+     'automatic_list_feature' => 'off',
+     'automatic_list_creation' => 'public',
+     'automatic_list_removal' => '', ## Can be 'if_empty'
      'global_remind' => 'listmaster',
      'wwsympa_url' => undef,
      'bounce_warn_rate' => '30',
@@ -191,6 +210,8 @@ my %Default_Conf =
      'list_check_suffixes' => 'request,owner,editor,unsubscribe,subscribe',
      'expire_bounce_task' => 'daily',
      'purge_user_table_task' => 'monthly',
+     'purge_logs_table_task' => 'daily',
+     'logs_expiration_period' => 3, #3 months
      'purge_orphan_bounces_task' => 'monthly',
      'eval_bouncers_task' => 'daily',
      'process_bouncers_task' => 'weekly',
@@ -206,19 +227,43 @@ my %Default_Conf =
      'default_bounce_level1_rate' => 45,
      'default_bounce_level2_rate' => 75,
      'soap_url' => '',
-     'css_url' => '',
-     'css_path' => '',
+     'css_url' => '', ## Defined below
+     'css_path' => '',## Defined below
      'urlize_min_size' => 10240, ## 10Kb
      'supported_lang' => 'de,cs,el,es,et_EE,en_US,fr,hu,it,ja_JP,nl,oc,pt_BR,sv,tr',
-     'web_recode_to' => '',
      'default_remind_task' => '',
      'update_db_field_types' => 'auto',
      'logo_html_definition' => '',
      'return_path_suffix' => '-owner',
-     'verp_rate' => '0%',
-     'cache_list_config' => 'none',
+     'verp_rate' => '0%', 
+     'pictures_max_size' => 102400, ## 100Kb
+     'pictures_feature' => 'off',
+     'use_blacklist' => 'send,subscribe',
+     'static_content_url' => '/static-sympa',
+     'static_content_path' => '--DIR--/static_content',
+     'filesystem_encoding' => 'utf-8',
+     'cache_list_config' => 'none', ## none | binary_file
+     'lock_method' => 'flock' ## flock | nfs
      );
    
+
+my %trusted_applications = ('trusted_application' => {'occurrence' => '0-n',
+						'format' => { 'name' => {'format' => '\S*',
+									 'occurrence' => '1',
+									 'case' => 'insensitive',
+								        },
+							      'ip'   => {'format' => '\d+\.\d+\.\d+\.\d+',
+									 'occurrence' => '0-1'},
+							      'md5password' => {'format' => '.*',
+										'occurrence' => '0-1'},
+							      'proxy_for_variables'=> {'format' => '.*',	    
+										      'occurrence' => '0-n',
+										      'split_char' => ','
+										  }
+							  }
+					    }
+			    );
+
 my $wwsconf;
 %Conf = ();
 
@@ -258,13 +303,16 @@ sub load {
     }
     close(IN);
 
+    ## Hardcoded values
+    foreach my $p (keys %hardcoded_options) {
+	$Ignored_Conf{$p} = $o{$p}[0] if (defined $o{$p});
+	$o{$p}[0] = $hardcoded_options{$p};
+    }
+
     ## Defaults
     unless (defined $o{'wwsympa_url'}) {
 	$o{'wwsympa_url'}[0] = "http://$o{'host'}[0]/wws";
     }
-    unless (defined $o{'css_url'}) {
-	$o{'css_url'}[0] = "$o{'wwsympa_url'}[0]/css/";
-    }
 
     # 'host' and 'domain' are mandatory and synonime.$Conf{'host'} is
     # still wydly use even if the doc require domain.
@@ -278,6 +326,10 @@ sub load {
 
     my $spool = $o{'spool'}[0] || $Default_Conf{'spool'};
 
+    unless (defined $o{'queueautomatic'}) {
+      $o{'queueautomatic'}[0] = "$spool/automatic";
+    }
+
     unless (defined $o{'queuedigest'}) {
 	$o{'queuedigest'}[0] = "$spool/digest";
     }
@@ -304,7 +356,7 @@ sub load {
     }
     unless (defined $o{'tmpdir'}) {
 	$o{'tmpdir'}[0] = "$spool/tmp";
-    }
+    }    
 
     ## Check if we have unknown values.
     foreach $i (sort keys %o) {
@@ -331,15 +383,35 @@ sub load {
 	$Conf{$i} = $o{$i}[0] || $Default_Conf{$i};
     }
 
+    ## Some parameters depend on others
+    unless ($Conf{'css_url'}) {
+	$Conf{'css_url'} = $Conf{'static_content_url'}.'/css';
+    }
     
+    unless ($Conf{'css_path'}) {
+	$Conf{'css_path'} = $Conf{'static_content_path'}.'/css';
+    }
+
+    ## Some parameters require CPAN modules
+    if ($Conf{'lock_method'} eq 'nfs') {
+	if (eval "require File::NFSLock") {
+	    require File::NFSLock;
+	}else {
+	    &do_log('err', "Failed to load File::NFSLock perl module ; setting 'lock_method' to 'flock'");
+	    $Conf{'lock_method'} = 'flock';
+	}
+    }
+
+    ## Load robot.conf files
     my $robots_conf = &load_robots ;    
     $Conf{'robots'} = $robots_conf ;
+
     my $nrcpt_by_domain =  &load_nrcpt_by_domain ;
     $Conf{'nrcpt_by_domain'} = $nrcpt_by_domain ;
     
     foreach my $robot (keys %{$Conf{'robots'}}) {
 	my $config;
-	unless ($config = &tools::get_filename('etc', 'auth.conf', $robot)) {
+	unless ($config = &tools::get_filename('etc',{},'auth.conf', $robot)) {
 	    &do_log('err',"_load_auth: Unable to find auth.conf");
 	    next;
 	}
@@ -373,6 +445,9 @@ sub load {
     }else {
 	$Conf{'rfc2369_header_fields'} = [split(/,/, $Conf{'rfc2369_header_fields'})];
     }
+    foreach my $action (split(/,/, $Conf{'use_blacklist'})) {
+	$Conf{'blacklist'}{$action} = 1;
+    }
 
     if ($Conf{'anonymous_header_fields'} eq 'none') {
 	delete $Conf{'anonymous_header_fields'};
@@ -397,6 +472,10 @@ sub load {
 
     $Conf{'sympa'} = "$Conf{'email'}\@$Conf{'host'}";
     $Conf{'request'} = "$Conf{'email'}-request\@$Conf{'host'}";
+    $Conf{'trusted_applications'} = &load_trusted_application (); 
+
+    $Conf{'pictures_url'}  = $Conf{'static_content_url'}.'/pictures/';
+    $Conf{'pictures_path'}  = $Conf{'static_content_path'}.'/pictures/';
     
     return 1;
 }
@@ -456,6 +535,9 @@ sub load_robots {
 				  log_smtp        => 1,
 				  log_level       => 1,
 				  create_list     => 1,
+				  automatic_list_feature     => 1,
+				  automatic_list_creation     => 1,
+				  automatic_list_removal     => 1,
 				  dark_color      => 1,
 				  light_color     => 1,
 				  text_color      => 1, 
@@ -469,9 +551,13 @@ sub load_robots {
 				  web_archive_spam_protection => 1,
 				  bounce_level1_rate => 1,
 				  bounce_level2_rate => 1,
+				  use_blacklist => 1,
 				  soap_url => 1,
 				  css_url => 1,
  				  css_path => 1,
+				  static_content_path => 1,
+                                  static_content_url => 1,
+				  pictures_max_size => 1,
  				  color_0 => 1, color_1 => 1, color_2 => 1, color_3 => 1, color_4 => 1, color_5 => 1,color_6 => 1, 
 				  color_7 => 1, color_8 => 1, color_9 => 1,
 				  color_10 => 1, color_11 => 1, color_12 => 1,color_13 => 1, color_14 => 1, color_15 => 1,
@@ -479,6 +565,7 @@ sub load_robots {
 				  default_shared_quota => 1,
 				  verp_rate => 1,
 				  loop_prevention_regex => 1,
+				  max_size => 1,
 				  );
 
     ## Load wwsympa.conf
@@ -498,7 +585,14 @@ sub load_robots {
 
     foreach $robot (readdir(DIR)) {
 	next unless (-d "$Conf{'etc'}/$robot");
-	next unless (-r "$Conf{'etc'}/$robot/robot.conf");
+	next unless (-f "$Conf{'etc'}/$robot/robot.conf");
+
+	unless (-r "$Conf{'etc'}/$robot/robot.conf") {
+	    printf STDERR "No read access on %s\n", "$Conf{'etc'}/$robot/robot.conf";
+	    &List::send_notify_to_listmaster('cannot_access_robot_conf',$Conf{'domain'}, ["No read access on $Conf{'etc'}/$robot/robot.conf. you should change privileges on this file to activate this virtual host. "]);
+	    next;
+	}
+
 	unless (open (ROBOT_CONF,"$Conf{'etc'}/$robot/robot.conf")) {
 	    printf STDERR "load robots config: Unable to open $Conf{'etc'}/$robot/robot.conf\n"; 
 	    next ;
@@ -540,12 +634,29 @@ sub load_robots {
 	$robot_conf->{$robot}{'log_smtp'} ||= $Conf{'log_smtp'};
 	$robot_conf->{$robot}{'log_level'} ||= $Conf{'log_level'};
 	$robot_conf->{$robot}{'wwsympa_url'} ||= 'http://'.$robot_conf->{$robot}{'http_host'}.'/wws';
-	$robot_conf->{$robot}{'css_url'} ||= $robot_conf->{$robot}{'wwsympa_url'}.'/css';
+
+	$robot_conf->{$robot}{'static_content_url'} ||= $Conf{'static_content_url'};
+	$robot_conf->{$robot}{'static_content_path'} ||= $Conf{'static_content_path'};
+
+	## CSS
+	$robot_conf->{$robot}{'css_url'} ||= $robot_conf->{$robot}{'static_content_url'}.'/css/'.$robot;
+	$robot_conf->{$robot}{'css_path'} ||= $robot_conf->{$robot}{'static_content_path'}.'/css/'.$robot;
+
 	$robot_conf->{$robot}{'sympa'} = $robot_conf->{$robot}{'email'}.'@'.$robot_conf->{$robot}{'host'};
 	$robot_conf->{$robot}{'request'} = $robot_conf->{$robot}{'email'}.'-request@'.$robot_conf->{$robot}{'host'};
 	$robot_conf->{$robot}{'cookie_domain'} ||= 'localhost';
 	#$robot_conf->{$robot}{'soap_url'} ||= $Conf{'soap_url'};
 	$robot_conf->{$robot}{'verp_rate'} ||= $Conf{'verp_rate'};
+	$robot_conf->{$robot}{'use_blacklist'} ||= $Conf{'use_blacklist'};
+
+	$robot_conf->{$robot}{'pictures_url'} ||= $robot_conf->{$robot}{'static_content_url'}.'/pictures/' if ($robot_conf->{$robot}{'static_content_url'}) ;
+	$robot_conf->{$robot}{'pictures_path'} ||= $robot_conf->{$robot}{'static_content_path'}.'/pictures/' if ($robot_conf->{$robot}{'static_content_path'});
+	$robot_conf->{$robot}{'pictures_feature'} ||= $Conf{'pictures_feature'};
+
+	# split action list for blacklist usage
+	foreach my $action (split(/,/, $Conf{'use_blacklist'})) {
+	    $robot_conf->{$robot}{'blacklist'}{$action} = 1;
+	}
 
 	my ($host, $path);
 	if ($robot_conf->{$robot}{'http_host'} =~ /^([^\/]+)(\/.*)$/) {
@@ -561,7 +672,8 @@ sub load_robots {
 	    $url =~ s/^http(s)?:\/\/(.+)$/$2/;
 	    $Conf{'robot_by_soap_url'}{$url} = $robot;
 	}
-
+	# printf STDERR "load trusted de $robot";
+	$robot_conf->{$robot}{'trusted_applications'} = &load_trusted_application($robot);
 	close (ROBOT_CONF);
     }
     closedir(DIR);
@@ -579,10 +691,12 @@ sub load_robots {
 ## Check required files and create them if required
 sub checkfiles_as_root {
 
+
     ## Check aliases file
     unless (-f $Conf{'sendmail_aliases'}) {
 	unless (open ALIASES, ">$Conf{'sendmail_aliases'}") {
 	    &do_log('err',"Failed to create aliases file %s", $Conf{'sendmail_aliases'});
+	    # printf STDERR "Failed to create aliases file %s", $Conf{'sendmail_aliases'};
 	    return undef;
 	}
 
@@ -596,6 +710,52 @@ sub checkfiles_as_root {
 	
     }
 
+    foreach my $robot (keys %{$Conf{'robots'}}) {
+
+	# create static content directory
+	my $dir = &get_robot_conf($robot, 'static_content_path');
+	if ($dir ne '' && ! -d $dir){
+	    unless ( mkdir ($dir, 0775)) {
+		&do_log('err', 'Unable to create directory %s : %s', $dir, $!);
+		printf STDERR 'Unable to create directory %s : $!',$dir, $!;
+		$config_err++;
+	    }
+
+	    # printf STDERR 'created directory %s',$Conf{'static_content_path'};
+	    `chown --USER-- $dir`;
+	    `chgrp --GROUP-- $dir`;
+	}
+    }
+
+    return 1 ;
+}
+
+## return 1 if the parameter is a known robot
+sub valid_robot {
+    my $robot = shift;
+
+    ## Main host
+    return 1 if ($robot eq $Conf{'domain'});
+
+    ## Missing etc directory
+    unless (-d $Conf{'etc'}.'/'.$robot) {
+	&do_log('err', 'Robot %s undefined ; no %s directory', $robot, $Conf{'etc'}.'/'.$robot);
+	return undef;
+    }
+
+    ## Missing expl directory
+    unless (-d $Conf{'home'}.'/'.$robot) {
+	&do_log('err', 'Robot %s undefined ; no %s directory', $robot, $Conf{'home'}.'/'.$robot);
+	return undef;
+    }
+    
+    ## Robot not loaded
+    unless (defined $Conf{'robots'}{$robot}) {
+	&do_log('err', 'Robot %s was not loaded by this Sympa process', $robot);
+	return undef;
+    }
+
+    return 1;
 }
 
 ## Check a few files
@@ -611,7 +771,7 @@ sub checkfiles {
 	}
     }
     
-    foreach my $qdir ('spool','queue','queuedigest','queuemod','queuetopic','queueauth','queueoutgoing','queuebounce','queuesubscribe','queuetask','queuedistribute','tmpdir')
+    foreach my $qdir ('spool','queue','queueautomatic','queuedigest','queuemod','queuetopic','queueauth','queueoutgoing','queuebounce','queuesubscribe','queuetask','queuedistribute','tmpdir')
     {
 	unless (-d $Conf{$qdir}) {
 	    do_log('info', "creating spool $Conf{$qdir}");
@@ -622,22 +782,17 @@ sub checkfiles {
 	}
     }
 
-    ## Also create msg/bad/
-    unless (-d $Conf{'queue'}.'/bad') {
-	    do_log('info', "creating spool $Conf{'queue'}/bad");
-	    unless ( mkdir ($Conf{'queue'}.'/bad', 0775)) {
-		do_log('err', 'Unable to create spool %s', $Conf{'queue'}.'/bad');
-		$config_err++;
-	    }
-	}
-    ## Also create distribute/bad/
-    unless (-d $Conf{'queuedistribute'}.'/bad') {
-	    do_log('info', "creating spool $Conf{'queuedistribute'}/bad");
-	    unless ( mkdir ($Conf{'queuedistribute'}.'/bad', 0775)) {
-		do_log('err', 'Unable to create spool %s', $Conf{'queuedistribute'}.'/bad');
+    ## Also create associated bad/ spools
+    foreach my $qdir ('queue','queuedistribute','queueautomatic')
+    {
+	unless (-d $Conf{$qdir}.'/bad') {
+	    do_log('info', "creating spool $Conf{$qdir}/bad");
+	    unless ( mkdir ($Conf{$qdir}.'/bad', 0775)) {
+		do_log('err', 'Unable to create spool %s', $Conf{$qdir}.'/bad');
 		$config_err++;
 	    }
 	}
+    }
 
     ## Check cafile and capath access
     if (defined $Conf{'cafile'} && $Conf{'cafile'}) {
@@ -669,10 +824,134 @@ sub checkfiles {
 	$config_err++;
     }
 
+    ## automatic_list_creation enabled but queueautomatic pointing to queue
+    if (($Conf{automatic_list_feature} eq 'on') && $Conf{'queue'} eq $Conf{'queueautomatic'}) {
+        &do_log('err', 'Error in config : queue and queueautomatic parameters pointing to the same directory (%s)', $Conf{'queue'});
+        unless (&List::send_notify_to_listmaster('queue_and_queueautomatic_are_the_same', $Conf{'domain'}, [$Conf{'queue'}])) {
+            &do_log('err', 'Unable to send notify "queue_and_queueautomatic_are_the_same" to listmaster');
+        }
+        $config_err++;
+    }
+
+    #  create pictures dir if usefull for each robot
+    foreach my $robot (keys %{$Conf{'robots'}}) {
+	my $dir = &get_robot_conf($robot, 'static_content_path');
+	if ($dir ne '' && -d $dir) {
+	    unless (-f $dir.'/index.html'){
+		unless(open (FF, ">$dir".'/index.html')) {
+		    &do_log('err', 'Unable to create %s/index.html as an empty file to protect directory : %s', $dir, $!);
+		}
+		close FF;		
+	    }
+	    
+	    # create picture dir
+	    if ( &get_robot_conf($robot, 'pictures_feature') eq 'on') {
+		my $pictures_dir = &get_robot_conf($robot, 'pictures_path');
+		unless (-d $pictures_dir){
+		    unless (mkdir ($pictures_dir, 0775)) {
+			do_log('err', 'Unable to create directory %s',$pictures_dir);
+			$config_err++;
+		    }
+		    chmod 0775, $pictures_dir;
+
+		    my $index_path = $pictures_dir.'/index.html';
+		    unless (-f $index_path){
+			unless (open (FF, ">$index_path")) {
+			    &do_log('err', 'Unable to create %s as an empty file to protect directory', $index_path);
+			}
+			close FF;
+		    }
+		}		
+	    }
+	}
+    }    		
+
+    # create or update static CSS files
+    my $css_updated = undef;
+    foreach my $robot (keys %{$Conf{'robots'}}) {
+	my $dir = &get_robot_conf($robot, 'css_path');
+	
+	## Get colors for parsing
+	my $param = {};
+	foreach my $p (@valid_options) {
+	    $param->{$p} = &Conf::get_robot_conf($robot, $p) if (($p =~ /_color$/)|| ($p =~ /color_/));
+	}
+
+	## Set TT2 path
+	my $tt2_include_path = &tools::make_tt2_include_path($robot,'web_tt2','','');
+
+	## Create directory if required
+	unless (-d $dir) {
+	    unless ( &tools::mkdir_all($dir, 0755)) {
+		&List::send_notify_to_listmaster('cannot_mkdir',  $robot, ["Could not create directory $dir : $!"]);
+		&do_log('err','Failed to create directory %s',$dir);
+		return undef;
+	    }
+	}
+
+	foreach my $css ('style.css','print.css','fullPage.css','print-preview.css') {
+
+	    $param->{'css'} = $css;
+
+	    ## Update the CSS if it is missing or if a new css.tt2 was installed
+	    if (! -f $dir.'/'.$css ||
+		(stat('--ETCBINDIR--/web_tt2/css.tt2'))[9] > (stat($dir.'/'.$css))[9]) {
+		&do_log('notice',"Updating static CSS file $dir/$css ; previous file renamed");
+		
+		## Keep copy of previous file
+		rename $dir.'/'.$css, $dir.'/'.$css.'.'.time;
+
+		unless (open (CSS,">$dir/$css")) {
+		    &List::send_notify_to_listmaster('cannot_open_file',  $robot, ["Could not open file $dir/$css : $!"]);
+		    &do_log('err','Failed to open (write) file %s',$dir.'/'.$css);
+		    return undef;
+		}
+		
+		unless (&tt2::parse_tt2($param,'css.tt2' ,\*CSS, $tt2_include_path)) {
+		    my $error = &tt2::get_error();
+		    $param->{'tt2_error'} = $error;
+		    &List::send_notify_to_listmaster('web_tt2_error', $robot, [$error]);
+		    &do_log('err', "Error while installing $dir/$css");
+		}
+
+		$css_updated ++;
+
+		close (CSS) ;
+		
+		## Make the CSS world-readable
+		chmod 0644, $dir.'/'.$css;
+	    }	    
+	}
+    }
+    if ($css_updated) {
+	## Notify main listmaster
+	&List::send_notify_to_listmaster('css_updated',  $Conf{'host'}, ["Static CSS files have been updated ; check log file for details"]);
+    }
+
+
     return undef if ($config_err);
     return 1;
 }
 
+## Returns the SSO record correponding to the provided sso_id
+## return undef if none was found
+sub get_sso_by_id {
+    my %param = @_;
+
+    unless (defined $param{'service_id'} && defined $param{'robot'}) {
+	return undef;
+    }
+
+    foreach my $sso (@{$Conf{'auth_services'}{$param{'robot'}}}) {
+	&do_log('notice', "SSO: $sso->{'service_id'}");
+	next unless ($sso->{'service_id'} eq $param{'service_id'});
+
+	return $sso;
+    }
+    
+    return undef;
+}
+
 ## Loads and parses the authentication configuration file.
 ##########################################
 
@@ -680,7 +959,7 @@ sub _load_auth {
     
     my $robot = shift;
     my $config = shift;
-    &do_log('notice', 'Conf::_load_auth(%s)', $config);
+    &do_log('debug', 'Conf::_load_auth(%s)', $config);
 
     my $line_num = 0;
     my $config_err = 0;
@@ -733,6 +1012,7 @@ sub _load_auth {
 					    'service_id' => '\S+',
 					    'http_header_prefix' => '\w+',
 					    'email_http_header' => '\w+',
+					    'logout_url' => '.+',
 					    'ldap_host' => '[\w\.\-]+(:\d+)?(\s*,\s*[\w\.\-]+(:\d+)?)*',
 					    'ldap_bind_dn' => '.+',
 					    'ldap_bind_password' => '.+',
@@ -863,5 +1143,291 @@ sub get_robot_conf {
     return $Conf{$param} || $wwsconf->{$param};
 }
 
+
+
+
+## load .sql named filter conf file
+sub load_sql_filter {
+	
+    my $file = shift;
+    my %sql_named_filter_params = (
+	'sql_named_filter_query' => {'occurrence' => '1',
+	'format' => { 
+		'db_type' => {'format' => 'mysql|SQLite|Pg|Oracle|Sybase', },
+		'db_name' => {'format' => '.*', 'occurrence' => '1', },
+		'db_host' => {'format' => '.*', 'occurrence' => '1', },
+		'statement' => {'format' => '.*', 'occurrence' => '1', },
+		'db_user' => {'format' => '.*', 'occurrence' => '0-1',  },
+		'db_passwd' => {'format' => '.*', 'occurrence' => '0-1',},
+		'db_options' => {'format' => '.*', 'occurrence' => '0-1',},
+		'db_env' => {'format' => '.*', 'occurrence' => '0-1',},
+		'db_port' => {'format' => '\d+', 'occurrence' => '0-1',},
+		'db_timeout' => {'format' => '\d+', 'occurrence' => '0-1',},
+	}
+	});
+
+    return undef unless  (-r $file);
+
+    return (&load_generic_conf_file($file,\%sql_named_filter_params, 'abort'));
+}
+
+## load trusted_application.conf configuration file
+sub load_trusted_application {
+    my $robot = shift;
+    
+    # find appropriate trusted-application.conf file
+    my $config ;
+    if (defined $robot) {
+	$config = $Conf{'etc'}.'/'.$robot.'/trusted_applications.conf';
+    }else{
+	$config = $Conf{'etc'}.'/trusted_applications.conf' ;
+    }
+    # print STDERR "load_trusted_applications $config ($robot)\n";
+
+    return undef unless  (-r $config);
+    # open TMP, ">/tmp/dump1";&tools::dump_var(&load_generic_conf_file($config,\%trusted_applications);, 0,\*TMP);close TMP;
+    return (&load_generic_conf_file($config,\%trusted_applications));
+
+
+}
+
+############################################################
+#  load_generic_conf_file
+############################################################
+#  load a generic config organized by paragraph syntax
+#  
+# IN : -$config_file (+): full path of config file
+#      -$structure_ref (+) : ref(HASH) describing expected syntax
+#      -$on_error : optional. sub returns undef if set to 'abort'
+#          and an error is found in conf file
+# OUT : ref(HASH) of parsed parameters
+#     | undef
+#
+############################################################## 
+sub load_generic_conf_file {
+    my $config_file = shift;
+    my $structure_ref = shift;
+    my $on_error = shift;
+    my %structure = %$structure_ref;
+
+    # printf STDERR "load_generic_file  $config_file \n";
+
+    unless (open (CONF,$config_file)) {
+	 printf STDERR "load_generic_conf_file: Unable to open $config_file";
+	 return undef;
+    }
+
+    my %admin;
+    my (@paragraphs);
+    
+    ## Just in case...
+    $/ = "\n";
+    
+    ## Set defaults to 1
+    foreach my $pname (keys %structure) {       
+	$admin{'defaults'}{$pname} = 1 unless ($structure{$pname}{'internal'});
+    }
+        ## Split in paragraphs
+    my $i = 0;
+    unless (open (CONFIG, $config_file)) {
+	printf STDERR 'unable to read configuration file %s\n',$config_file;
+	return undef;
+    }
+    while (<CONFIG>) {
+	if (/^\s*$/) {
+	    $i++ if $paragraphs[$i];
+	}else {
+	    push @{$paragraphs[$i]}, $_; # printf STDERR "xxxxxxxxxxxxxxxxxxx detected paragraph $_\n";
+	}
+    }
+   # printf STDERR "xxxxxxxxxxxxxxxxxxx #detected paragraph $i\n";
+
+    for my $index (0..$#paragraphs) {
+	my @paragraph = @{$paragraphs[$index]};
+
+	my $pname;
+
+	# printf STDERR "xxxxxxxxxxxxxxxxxxx paragraph $index\n";
+	## Clean paragraph, keep comments
+	for my $i (0..$#paragraph) {
+	    my $changed = undef;
+	    for my $j (0..$#paragraph) {
+		if ($paragraph[$j] =~ /^\s*\#/) {
+		    chomp($paragraph[$j]);
+		    push @{$admin{'comment'}}, $paragraph[$j];
+		    splice @paragraph, $j, 1;
+		    $changed = 1;
+		}elsif ($paragraph[$j] =~ /^\s*$/) {
+		    splice @paragraph, $j, 1;
+		    $changed = 1;
+		}
+
+		last if $changed;
+	    }
+
+	    last unless $changed;
+	}
+
+	## Empty paragraph
+	next unless ($#paragraph > -1);
+	
+	## Look for first valid line
+	unless ($paragraph[0] =~ /^\s*([\w-]+)(\s+.*)?$/) {
+	    printf STDERR 'Bad paragraph "%s" in %s, ignored', @paragraph, $config_file;
+	    return undef if $on_error eq 'abort';
+	    next;
+	}
+	    
+	$pname = $1;	
+	# printf STDERR "xxxxxxxxxxxx parametre pname $pname\n";
+	unless (defined $structure{$pname}) {
+	    printf STDERR 'Unknown parameter "%s" in %s, ignored', $pname, $config_file;
+	    return undef if $on_error eq 'abort';
+	    next;
+	}
+	## Uniqueness
+	if (defined $admin{$pname}) {
+	    unless (($structure{$pname}{'occurrence'} eq '0-n') or
+		    ($structure{$pname}{'occurrence'} eq '1-n')) {
+		printf STDERR 'Multiple parameter "%s" in %s', $pname, $config_file;
+		return undef if $on_error eq 'abort';
+	    }
+	}
+	
+	## Line or Paragraph
+	if (ref $structure{$pname}{'format'} eq 'HASH') {
+	    ## This should be a paragraph
+	    unless ($#paragraph > 0) {
+		printf STDERR 'Expecting a paragraph for "%s" parameter in %s, ignore it\n', $pname, $config_file;
+		return undef if $on_error eq 'abort';
+		next;
+	    }
+	    
+	    ## Skipping first line
+	    shift @paragraph;
+
+	    my %hash;
+	    for my $i (0..$#paragraph) {	    
+		next if ($paragraph[$i] =~ /^\s*\#/);		
+		unless ($paragraph[$i] =~ /^\s*(\w+)\s*/) {
+		    printf STDERR 'Bad line "%s" in %s\n',$paragraph[$i], $config_file;
+		    return undef if $on_error eq 'abort';
+		}		
+		my $key = $1;
+			
+		unless (defined $structure{$pname}{'format'}{$key}) {
+		    printf STDERR 'Unknown key "%s" in paragraph "%s" in %s\n', $key, $pname, $config_file;
+		    return undef if $on_error eq 'abort';
+		    next;
+		}
+		
+		unless ($paragraph[$i] =~ /^\s*$key\s+($structure{$pname}{'format'}{$key}{'format'})\s*$/i) {
+		    printf STDERR 'Bad entry "%s" in paragraph "%s" in %s\n', $paragraph[$i], $key, $pname, $config_file;
+		    return undef if $on_error eq 'abort';
+		    next;
+		}
+
+		$hash{$key} = &_load_a_param($key, $1, $structure{$pname}{'format'}{$key});
+	    }
+
+
+	    ## Apply defaults & Check required keys
+	    my $missing_required_field;
+	    foreach my $k (keys %{$structure{$pname}{'format'}}) {
+
+		## Default value
+		unless (defined $hash{$k}) {
+		    if (defined $structure{$pname}{'format'}{$k}{'default'}) {
+			$hash{$k} = &_load_a_param($k, 'default', $structure{$pname}{'format'}{$k});
+		    }
+		}
+
+		## Required fields
+		if ($structure{$pname}{'format'}{$k}{'occurrence'} eq '1') {
+		    unless (defined $hash{$k}) {
+			printf STDERR 'Missing key %s in param %s in %s\n', $k, $pname, $config_file;
+			return undef if $on_error eq 'abort';
+			$missing_required_field++;
+		    }
+		}
+	    }
+
+	    next if $missing_required_field;
+
+	    delete $admin{'defaults'}{$pname};
+
+	    ## Should we store it in an array
+	    if (($structure{$pname}{'occurrence'} =~ /n$/)) {
+		push @{$admin{$pname}}, \%hash;
+	    }else {
+		$admin{$pname} = \%hash;
+	    }
+	}else {
+	    ## This should be a single line
+	    my $xxxmachin =  $structure{$pname}{'format'};
+	    unless ($#paragraph == 0) {
+		printf STDERR 'Expecting a single line for %s parameter in %sxxxxxx %s\n', $pname, $config_file, $xxxmachin ;
+		return undef if $on_error eq 'abort';
+	    }
+
+	    unless ($paragraph[0] =~ /^\s*$pname\s+($structure{$pname}{'format'})\s*$/i) {
+		printf STDERR 'Bad entry "%s" in %s\n', $paragraph[0], $config_file ;
+		return undef if $on_error eq 'abort';
+		next;
+	    }
+
+	    my $value = &_load_a_param($pname, $1, $structure{$pname});
+
+	    delete $admin{'defaults'}{$pname};
+
+	    if (($structure{$pname}{'occurrence'} =~ /n$/)
+		&& ! (ref ($value) =~ /^ARRAY/)) {
+		push @{$admin{$pname}}, $value;
+	    }else {
+		$admin{$pname} = $value;
+	    }
+	}
+    }
+    
+    close CONFIG;
+#open TMP2, ">>/tmp/sss"; printf TMP2 "xxxxxxxxxxxxxxxxxxx--------structure admin\n"; &tools::dump_var(\%admin, 0, \*TMP2);printf TMP2 "xxxxxxxxxxxxxxxxxxx--------\n"; close TMP2;
+    return \%admin;
+}
+
+
+### load_a_param
+# 
+sub _load_a_param {
+    my ($key, $value, $p) = @_;
+    # print STDERR 'xxxxxxxxxxxxx _load_list_param(%s,\'%s\',\'%s\')\n', $robot,$key, $value;
+    
+    ## Empty value
+    if ($value =~ /^\s*$/) {
+	return undef;
+    }
+    
+    ## Default
+    if ($value eq 'default') {
+	$value = $p->{'default'};
+    }
+    ## lower case if usefull
+    $value = lc($value) if ($p->{'case'} eq 'insensitive'); 
+    
+    ## Do we need to split param if it is not already an array
+    if (($p->{'occurrence'} =~ /n$/)
+	&& $p->{'split_char'}
+	&& !(ref($value) eq 'ARRAY')) {
+	my @array = split /$p->{'split_char'}/, $value;
+	foreach my $v (@array) {
+	    $v =~ s/^\s*(.+)\s*$/$1/g;
+	}
+	
+	return \@array;
+    }else {
+	return $value;
+    }
+}
+
+
 ## Packages must return true.
 1;
