@@ -1,5 +1,5 @@
 # Language.pm - This module does just the initial setup for the international messages
-# RCS Identication ; $Revision: 1.58.2.2 $ ; $Date: 2006/06/08 12:53:26 $ 
+# RCS Identication ; $Revision: 4598 $ ; $Date: 2007-09-13 16:28:01 +0200 (jeu, 13 sep 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -24,7 +24,7 @@ package Language;
 require Exporter;
 use Carp;
 @ISA = qw(Exporter);
-@EXPORT = qw(&gettext);
+@EXPORT = qw(&gettext gettext_strftime);
 
 use strict;
 use Log;
@@ -43,7 +43,7 @@ my %set_comment; #sets-of-messages comment
 
 ## The lang is the NLS catalogue name ; locale is the locale preference
 ## Ex: lang = fr ; locale = fr_FR
-my ($current_lang, $current_locale, @previous_locale);
+my ($current_lang, $current_locale, $current_charset, @previous_locale);
 my $default_lang;
 ## This was the old style locale naming, used for templates, nls, scenario
 my %language_equiv = ( 'zh_CN' => 'cn',
@@ -60,6 +60,7 @@ my %lang2locale = ('cz' => 'cs_CZ',
 		   'el' => 'el_GR',
 		   'es' => 'es_ES',
 		   'et' => 'et_EE',
+		   'eu' => 'eu_ES',
 		   'fi' => 'fi_FI',
 		   'fr' => 'fr_FR',
 		   'hu' => 'hu_HU',
@@ -77,32 +78,22 @@ my %lang2locale = ('cz' => 'cs_CZ',
 		   'tr' => 'tr_TR',
 		   'tw' => 'zh_TW');
 
-## Used to perform setlocale on FreeBSD / Solaris
-my %locale2charset = ('cs_CZ' => 'utf-8',
-		      'de_DE' => 'iso8859-1',
-		      'el_GR' => 'utf-8',
-		      'en_US' => 'utf-8',
-		      'es_ES' => 'iso8859-1',
-		      'et_EE' => 'iso8859-4',
-		      'fi_FI' => 'iso8859-1',
-		      'fr_FR' => 'iso8859-1',
-		      'hu_HU' => 'iso8859-2',
-		      'it_IT' => 'iso8859-1',
-		      'ja_JP' => 'utf-8',
-		      'nb_NO' => 'utf-8',
-		      'nl_NL' => 'iso8859-1',
-		      'oc_FR' => 'iso8859-1',		      
-		      'pl_PL' => 'iso8859-2',
-		      'pt_BR' => 'utf-8',
-		      'pt_PT' => 'iso8859-1',
-		      'ro_RO' => 'iso8859-2',
-		      'sv_SE' => 'utf-8',
-		      'tr_TR' => 'utf-8',
-		      'zh_CN' => 'utf-8',
-		      'zh_TW' => 'big5',
-		      );
-
-my $recode;
+## We use different catalog/textdomains depending on the template that requests translations
+my %template2textdomain = ('help_admin.tt2' => 'web_help',
+			   'help_arc.tt2' => 'web_help',
+			   'help_editfile.tt2' => 'web_help',
+			   'help_editlist.tt2' => 'web_help',
+			   'help_faqadmin.tt2' => 'web_help',
+			   'help_faquser.tt2' => 'web_help',
+			   'help_introduction.tt2' => 'web_help',
+			   'help_listconfig.tt2' => 'web_help',
+			   'help_mail_commands.tt2' => 'web_help',
+			   'help_sendmsg.tt2' => 'web_help',
+			   'help_shared.tt2' => 'web_help',
+			   'help.tt2' => 'web_help',
+			   'help_user_options.tt2' => 'web_help',
+			   'help_user.tt2' => 'web_help',
+			   );			   
 
 sub GetSupportedLanguages {
     my $robot = shift;
@@ -159,12 +150,12 @@ sub SetLang {
     }
    
     ## Set Locale::Messages context
-    my $locale_dashless = $locale.'.'.$locale2charset{$locale}; 
+    my $locale_dashless = $locale.'.utf-8';
     $locale_dashless =~ s/-//g;
     foreach my $type (&POSIX::LC_ALL, &POSIX::LC_TIME) {
 	my $success;
-	foreach my $try ($locale.'.'.$locale2charset{$locale},
-			 $locale.'.'.uc($locale2charset{$locale}),  ## UpperCase required for FreeBSD
+	foreach my $try ($locale.'.utf-8',
+			 $locale.'.UTF-8',  ## UpperCase required for FreeBSD
 			 $locale_dashless, ## Required on HPUX
 			 $locale,
 			 $lang
@@ -180,20 +171,34 @@ sub SetLang {
 	}
     }
     
+#    &Locale::Messages::bindtextdomain('web_help','--LOCALEDIR--');
+#    bind_textdomain_codeset web_help => 'utf-8';
+
+    $ENV{'LANGUAGE'}=$locale;
     ## Define what catalog is used
     &Locale::Messages::textdomain("sympa");
     &Locale::Messages::bindtextdomain('sympa','--LOCALEDIR--');
-    &Locale::Messages::bind_textdomain_codeset('sympa',$recode) if $recode;
-    #bind_textdomain_codeset sympa => 'iso-8859-1';
+    # Get translations by internal encoding.
+    bind_textdomain_codeset sympa => 'utf-8';
 
     $current_lang = $lang;
     $current_locale = $locale;
+    $current_charset = 'utf-8';
 
     return $locale;
 }#SetLang
 
-sub set_recode {
-    $recode = shift;
+
+## Get the name of the language, ie the one defined in the catalog
+sub GetLangName {
+    my $lang = shift;
+
+    my $saved_lang = $current_lang;
+    &SetLang($lang);
+    my $name = gettext('_language_');
+    &SetLang($saved_lang);
+    
+    return $name;
 }
 
 sub GetLang {
@@ -202,6 +207,11 @@ sub GetLang {
     return $current_lang;
 }
 
+sub GetCharset {
+
+    return $current_charset;
+}
+
 sub Locale2Lang {
     my $locale = shift;
     my $lang;
@@ -224,11 +234,20 @@ sub Lang2Locale {
 }
 
 sub maketext {
+    my $template_file = shift;
     my $msg = shift;
 
 #    &do_log('notice','Maketext: %s', $msg);
 
-    my $translation = &gettext ($msg);
+    my $translation;
+    my $textdomain = $template2textdomain{$template_file};
+    
+#    if ($textdomain) {
+#	$translation = &sympa_dgettext ($textdomain, $msg);
+#    }else {
+#	$translation = &gettext ($msg);
+#    }
+    $translation = &gettext ($msg);
 
     ## replace parameters in string
     $translation =~ s/\%\%/'_ESCAPED_'.'%_'/eg; ## First escape '%%'
@@ -238,27 +257,30 @@ sub maketext {
     return $translation;
 }
 
-sub gettext {
-    &do_log('debug3', 'Language::gettext(%s)', $_[0]);
+
+sub sympa_dgettext {
+    my $textdomain = shift;
+    my @param = @_;
+
+    &do_log('debug3', 'Language::gettext(%s)', $param[0]);
 
     ## This prevents meta information to be returned if the string to translate is empty
-    if ($_[0] eq '') {
+    if ($param[0] eq '') {
 	return '';
 	
 	## return meta information on the catalogue (language, charset, encoding,...)
-    }elsif ($_[0] =~ '^_(\w+)_$') {
+    }elsif ($param[0] =~ '^_(\w+)_$') {
 	my $var = $1;
 	foreach (split /\n/,&Locale::Messages::gettext('')) {
 	    if ($var eq 'language') {
 		if (/^Language-Team:\s*(.+)$/i) {
 		    my $language = $1;
 		    $language =~ s/\<\S+\>//;
+
 		    return $language;
 		}
 	    }elsif ($var eq 'charset') {
-		if ($recode) {
-		    return $recode;
-		} elsif (/^Content-Type:\s*.*charset=(\S+)$/i) {
+		if (/^Content-Type:\s*.*charset=(\S+)$/i) {
 		    return $1;
 		}
 	    }elsif ($var eq 'encoding') {
@@ -270,7 +292,56 @@ sub gettext {
 	return '';
     }
 
-    &Locale::Messages::gettext(@_);
+    return &Locale::Messages::dgettext($textdomain, @param);
+
+}
+
+sub gettext {
+    my @param = @_;
+
+    &do_log('debug3', 'Language::gettext(%s)', $param[0]);
+
+    ## This prevents meta information to be returned if the string to translate is empty
+    if ($param[0] eq '') {
+	return '';
+	
+	## return meta information on the catalogue (language, charset, encoding,...)
+    }elsif ($param[0] =~ '^_(\w+)_$') {
+	my $var = $1;
+	foreach (split /\n/,&Locale::Messages::gettext('')) {
+	    if ($var eq 'language') {
+		if (/^Language-Team:\s*(.+)$/i) {
+		    my $language = $1;
+		    $language =~ s/\<\S+\>//;
+
+		    return $language;
+		}
+	    }elsif ($var eq 'charset') {
+		if (/^Content-Type:\s*.*charset=(\S+)$/i) {
+		    return $1;
+		}
+	    }elsif ($var eq 'encoding') {
+		if (/^Content-Transfer-Encoding:\s*(.+)$/i) {
+		    return $1;
+		}
+	    }
+	}
+	return '';
+    }
+
+    return &Locale::Messages::gettext(@param);
+
+}
+
+sub gettext_strftime {
+    my $format = shift;
+    return &POSIX::strftime($format, @_) unless $current_charset;
+
+    $format = gettext($format);
+    Encode::from_to($format, 'utf8', $current_charset);
+    my $datestr = &POSIX::strftime($format, @_);
+    Encode::from_to($datestr, $current_charset, 'utf8');
+    return $datestr;
 }
 
 1;
