@@ -1,5 +1,5 @@
 # mail.pm - This module includes mail sending functions and does the smtp job.
-# RCS Identication ; $Revision: 1.35.2.1 $ ; $Date: 2007/01/22 15:13:33 $ 
+# RCS Identication ; $Revision: 4479 $ ; $Date: 2007-06-26 18:09:52 +0200 (mar, 26 jun 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -29,6 +29,9 @@ use Carp;
 #use strict;
 use POSIX;
 use Mail::Internet;
+use MIME::Charset;
+use MIME::Tools;
+use Version;
 use Conf;
 use Log;
 use Language;
@@ -39,7 +42,7 @@ require 'tools.pl';
 #use strict;
 
 ## RCS identification.
-#my $id = '@(#)$Id: mail.pm,v 1.35.2.1 2007/01/22 15:13:33 olivier.salaun Exp $';
+#my $id = '@(#)$Id: mail.pm 4479 2007-06-26 16:09:52Z olivier.salaun $';
 
 my $opensmtp = 0;
 my $fh = 'fh0000000000';	## File handle for the stream.
@@ -122,6 +125,9 @@ sub mail_file {
 	}
     }
 
+    ## Charset for encoding
+    $data->{'charset'} ||= &Language::GetCharset();
+
     ## TT2 file parsing 
     if ($filename =~ /\.tt2$/) {
 	my $output;
@@ -141,8 +147,10 @@ sub mail_file {
 	foreach my $line (split(/\n/,$message)) {
 	    last if ($line=~/^\s*$/);
        
-	    if ($line=~/^[\w-]+:\s+\S/) {
+	    if ($line=~/^[\w-]+:\s*/) { ## A header field
 		$existing_headers=1;
+	    }elsif ($existing_headers && ($line =~ /^\s/)) { ## Following of a header field
+		next;
 	    }else{
 		last;
 	    }
@@ -156,9 +164,6 @@ sub mail_file {
 	}
    }
    
-   ## Charset for encoding
-   my $charset = sprintf (gettext("_charset_"));
-
     ## ADD MISSING HEADERS
     my $headers="";
 
@@ -167,48 +172,78 @@ sub mail_file {
 	if (ref ($rcpt)) {
 	    if ($data->{'to'}) {
 		$to = $data->{'to'};
-   }else {
+	    }else {
 		$to = join(",\n   ", @{$rcpt});
 	    }
 	}else{
 	    $to = $rcpt;
 	}   
-	$headers .= "To: ".MIME::Words::encode_mimewords($to, ('Encode' => 'Q', 'Charset' => $charset))."\n"; 
+	$headers .= "To: ".MIME::EncWords::encode_mimewords(
+	    Encode::decode('utf8', $to),
+	    'Encoding' => 'A', 'Charset' => $data->{'charset'}, 'Field' => 'To'
+	    )."\n"; 
     }     
     unless ($header_ok{'from'}) {
 	if ($data->{'from'} eq 'sympa') {
-	    $headers .= "From: ".MIME::Words::encode_mimewords((sprintf ("SYMPA <%s>",&Conf::get_robot_conf($robot, 'sympa'))), ('Encode' => 'Q', 'Charset' => $charset))."\n";
+	    $headers .= "From: ".MIME::EncWords::encode_mimewords(
+		sprintf("SYMPA <%s>",&Conf::get_robot_conf($robot, 'sympa')),
+		'Encoding' => 'A', 'Charset' => "US-ASCII", 'Field' => 'From'
+		)."\n";
 	} else {
-	    $headers .= "From: ".MIME::Words::encode_mimewords($data->{'from'},('Encode' => 'Q', 'Charset' => $charset))."\n"; 
+	    $headers .= "From: ".MIME::EncWords::encode_mimewords(
+		Encode::decode('utf8', $data->{'from'}),
+		'Encoding' => 'A', 'Charset' => $data->{'charset'}, 'Field' => 'From'
+		)."\n"; 
 	}
    }
     unless ($header_ok{'subject'}) {
-	$headers .= "Subject: ".MIME::Words::encode_mimewords($data->{'subject'},('Encode' => 'Q', 'Charset' => $charset))."\n";
+	$headers .= "Subject: ".MIME::EncWords::encode_mimewords(
+	    Encode::decode('utf8', $data->{'subject'}),
+	    'Encoding' => 'A', 'Charset' => $data->{'charset'}, 'Field' => 'Subject'
+	    )."\n";
    }
     unless ($header_ok{'reply-to'}) { 
-	$headers .= "Reply-to: ".MIME::Words::encode_mimewords($data->{'replyto'},('Encode' => 'Q', 'Charset' => $charset))."\n" if ($data->{'replyto'})
+	$headers .= "Reply-to: ".MIME::EncWords::encode_mimewords(
+	    Encode::decode('utf8', $data->{'replyto'}),
+	    'Encoding' => 'A', 'Charset' => $data->{'charset'}, 'Field' => 'Reply-to'
+	    )."\n" if ($data->{'replyto'})
     }
     if ($data->{'headers'}) {
 	foreach my $field (keys %{$data->{'headers'}}) {
-	    $headers .= $field.': '.MIME::Words::encode_mimewords($data->{'headers'}{$field},('Encode' => 'Q', 'Charset' => $charset))."\n";
+	    $headers .= $field.': '.MIME::EncWords::encode_mimewords(
+		Encode::decode('utf8', $data->{'headers'}{$field}),
+		'Encoding' => 'A', 'Charset' => $data->{'charset'}, 'Field' => $field
+		)."\n";
 	}
     }
     unless ($header_ok{'mime-version'}) {
 	$headers .= "MIME-Version: 1.0\n";
     }
     unless ($header_ok{'content-type'}) {
-	$headers .= "Content-Type: text/plain; charset=$charset\n";
+	$headers .= "Content-Type: text/plain; charset=UTF-8\n";
     }
     unless ($header_ok{'content-transfer-encoding'}) {
-	$headers .= "Content-Transfer-Encoding:"; 
-        $headers .= gettext("_encoding_");
-	$headers .= "\n";
+	$headers .= "Content-Transfer-Encoding: 8bit\n"; 
     }
     unless ($existing_headers) {
 	$headers .= "\n";
    }
    
-    $message = "$headers"."$message";
+    ## All these data provide mail attachements in service messages
+    my @msgs = ();
+    if (ref($data->{'msg_list'}) eq 'ARRAY') {
+	@msgs = map {$_->{'msg'} || $_->{'full_msg'}} @{$data->{'msg_list'}};
+    } elsif ($data->{'spool'}) {
+	@msgs = @{$data->{'spool'}};
+    } elsif ($data->{'msg'}) {
+	push @msgs, $data->{'msg'};
+    } elsif ($data->{'msg_path'} and open IN, '<'.$data->{'msg_path'}) {
+	push @msgs, join('', <IN>);
+	close IN;
+    } elsif ($data->{'file'} and open IN, '<'.$data->{'file'}) {
+	push @msgs, join('', <IN>);
+	close IN;
+    }
 
     my $listname = '';
     if (ref($data->{'list'}) eq "HASH") {
@@ -217,6 +252,10 @@ sub mail_file {
 	$listname = $data->{'list'};
     }
      
+    unless ($message = &reformat_message("$headers"."$message", \@msgs)) {
+	&do_log('err', "mail::mail_file: Failed to reformat message");
+    }
+
     ## Set it in case it was not set
     $data->{'return_path'} ||= &Conf::get_robot_conf($robot, 'request');
   
@@ -446,8 +485,8 @@ sub sendto {
 
     my $msg;
 
-    ## Encode subject before sending
-    $msg_header->replace('Subject', MIME::Words::encode_mimewords($msg_header->get('Subject')));
+    #XXX## Encode subject before sending
+    #XXX$msg_header->replace('Subject', MIME::Words::encode_mimewords($msg_header->get('Subject')));
 
     if ($encrypt eq 'smime_crypted') {
 	my $email ;
@@ -522,6 +561,11 @@ sub sending {
 	    $all_rcpt = $$rcpt;
 	}
 	
+	unless ($all_rcpt && $from) {
+	    &do_log('err', 'mail::sending: missing required parameter, cannot send message');
+	    return undef;
+	}
+
 	unless (open TMP, ">$sympa_file") {
 	    &do_log('notice', 'mail::sending : Cannot create %s : %s', $sympa_file, $!);
 	    return undef;
@@ -744,13 +788,129 @@ sub send_in_spool {
 
 #####################################################################
 
+
+####################################################
+# reformat_message
+####################################################
+# Reformat bodies of text parts contained in the message using
+# recommended encoding schema and/or charsets defined by MIME::Charset.
+#
+# MIME-compliant headers are appended / modified.  And custom X-Mailer:
+# header is appended :).
+#
+# IN : $msg: ref(MIME::Entity) | string - message to reformat
+#      $attachments: ref(ARRAY) - messages to be attached as subparts.
+# OUT : string
+#
+####################################################
+
+sub reformat_message($;$) {
+    my $message = shift;
+    my $attachments = shift || [];
+    my $msg;
+
+    my $parser = new MIME::Parser;
+    unless (defined $parser) {
+	&do_log('err', "mail::reformat_message: Failed to create MIME parser");
+	return undef;
+    }
+    $parser->output_to_core(1);
+
+    if (ref($message) eq 'MIME::Entity') {
+	$msg = $message;
+    } else {
+	eval {
+	    $msg = $parser->parse_data($message);
+	};
+	if ($@) {
+	    &do_log('err', "mail::reformat_message: Failed to parse MIME data");
+	    return undef;
+	}
+    }
+
+    $msg->head->delete("X-Mailer");
+    $msg = &fix_part($msg, $parser, $attachments);
+    $msg->head->add("X-Mailer", sprintf "Sympa %s", $Version::Version);
+    return $msg->as_string;
+}
+
+sub fix_part($$$) {
+    my $part = shift;
+    my $parser = shift;
+    my $attachments = shift || [];
+    return $part unless $part;
+
+    my $enc = $part->head->mime_attr("Content-Transfer-Encoding");
+    # Parts with nonstandard encodings aren't modified.
+    return $part
+	if $enc and $enc !~ /^(?:base64|quoted-printable|[78]bit|binary)$/i;
+
+    my $eff_type = $part->effective_type;
+    return $part if $eff_type =~ m{^multipart/(signed|encrypted)$};
+
+    if ($part->head->get('X-Sympa-Attach')) { # Need re-attaching data.
+	my $data = shift @{$attachments};
+	if (ref($data) ne 'MIME::Entity') {
+	    eval {
+		$data = $parser->parse_data($data);
+	    };
+	    if ($@) {
+		&do_log('warning',
+			"mail::reformat_message: Failed to parse MIME data");
+		$data = $parser->parse_data('');
+	    }
+	}
+	$part->head->delete('X-Sympa-Attach');
+	$part->parts([$data]);
+    } elsif ($part->parts) {
+	my @newparts = ();
+	foreach ($part->parts) {
+	    push @newparts, &fix_part($_, $parser, $attachments);
+	}
+	$part->parts(\@newparts);
+    } elsif ($eff_type =~ m{^(?:multipart|message)(?:/|\Z)}i) {
+	# multipart or message types without subparts.
+	return $part;
+    } elsif (MIME::Tools::textual_type($eff_type)) {
+	my $bodyh = $part->bodyhandle;
+	# Encoded body or null body won't be modified.
+	return $part if !$bodyh or $bodyh->is_encoded;
+
+	my $head = $part->head;
+	my $body = $bodyh->as_string;
+	my $charset = $head->mime_attr("Content-Type.Charset");
+
+	my ($newbody, $newcharset, $newenc) = 
+	    MIME::Charset::body_encode(Encode::decode('utf8', $body), $charset,
+				       Replacement => 'FALLBACK');
+	if ($newenc eq $enc and $newcharset eq $charset and
+	    $newbody eq $body) {
+	    return $part;
+	}
+
+	# Fix headers and body.
+	$head->mime_attr("Content-Type", "TEXT/PLAIN")
+	    unless $head->mime_attr("Content-Type");
+	$head->mime_attr("Content-Type.Charset", $newcharset);
+	$head->mime_attr("Content-Transfer-Encoding", $newenc);
+	$head->add("MIME-Version", "1.0") unless $head->get("MIME-Version"); 
+	my $io = $bodyh->open("w");
+
+	unless (defined $io) {
+	    &do_log('err', "mail::reformat_message: Failed to save message : $!");
+	    return undef;
+	}
+
+	$io->print($newbody);
+	$io->close;
+	$part->sync_headers(Length => 'COMPUTE');
+    } else {
+	# Binary or text with long lines will be suggested to be BASE64.
+	$part->head->mime_attr("Content-Transfer-Encoding",
+			       $part->suggest_encoding);
+	$part->sync_headers(Length => 'COMPUTE');
+    }
+    return $part;
+}
+
 1;
-
-
-
-
-
-
-
-
-
