@@ -1,5 +1,5 @@
 /* queue.c - This program does the messages spooling
-   RCS Identication ; $Revision: 1.7 $ ; $Date: 2006/03/21 09:59:04 $ 
+   RCS Identication ; $Revision: 4399 $ ; $Date: 2007-05-15 14:01:14 +0200 (mar, 15 mai 2007) $ 
 
    Sympa - SYsteme de Multi-Postage Automatique
    Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -26,7 +26,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-static char rcsid[] = "(@)$Id: queue.c,v 1.7 2006/03/21 09:59:04 sympa-authors Exp $";
+static char rcsid[] = "(@)$Id: queue.c 4399 2007-05-15 12:01:14Z olivier.salaun $";
 
 static char     qfile[128];
 static char     buf[16384];
@@ -87,7 +87,7 @@ main(int argn, char **argv)
    char	*queuedir;
    char        *listname;
    unsigned int		priority;
-   int			firstfrom = 0;
+   int			firstline = 1;
 
    /* Usage : queue list-name */
    if ((argn < 2) || (argn >3)) {
@@ -133,10 +133,11 @@ main(int argn, char **argv)
    write(fd, listname, strlen(listname));
    write(fd, "\n", 1);
    while (fgets(buf, sizeof buf, stdin) != NULL) {
-      if (firstfrom == 0 && strncmp(buf, "From ", 5) == 0) {
-         firstfrom = 1;
+      if (firstline == 1 && strncmp(buf, "From ", 5) == 0) {
+         firstline = 0;
          continue;
       }
+      firstline = 0;
       write(fd, buf, strlen(buf));
    }
    while ((i = read(fileno(stdin), buf, sizeof buf)) > 0)
