@@ -1,6 +1,6 @@
 # report.pm - This module provides various tools for command and message 
 # diffusion report
-# RCS Identication ; $Revision: 1.4 $ ; $Date: 2006/02/07 08:30:21 $ 
+# RCS Identication ; $Revision: 3999 $ ; $Date: 2006-11-03 15:57:57 +0100 (ven, 03 nov 2006) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -59,6 +59,7 @@ use List;
 ############################################################## 
 sub reject_report_msg {
     my ($type,$error,$user,$param,$robot,$msg_string,$list) = @_;
+    &do_log('debug2', "reject::reject_report_msg(%s,%s,%s)", $type,$error,$user);
 
     unless ($type eq 'intern' || $type eq 'intern_quiet' || $type eq 'user'|| $type eq 'auth') {
 	&do_log('err',"report::reject_report_msg(): error to prepare parsing 'message_report' template to $user : not a valid error type");
@@ -91,6 +92,11 @@ sub reject_report_msg {
 	$param->{'type'} = 'intern_error';
     }
 
+    ## Prepare the original message if provided
+    if (defined $param->{'message'}) {
+	$param->{'original_msg'} = &_get_msg_as_hash($param->{'message'});
+     }
+
     if (ref($list) eq "List") {
 	unless ($list->send_file('message_report',$user,$robot,$param)) {
 	    &do_log('notice',"report::reject_report_msg(): Unable to send template 'message_report' to '$user'");
@@ -121,6 +127,52 @@ sub reject_report_msg {
 }
 
 
+
+############################################################
+#  _get_msg_as_hash
+############################################################
+#  Internal subroutine
+#  Provide useful parts of a message as a hash entries
+#  
+# IN : -$msg_object (+): ref(HASH) - the MIME::Entity or Message object
+#
+# OUT : $msg_hash : ref(HASH) - the hashref
+#
+############################################################## 
+
+sub _get_msg_as_hash {
+    my $msg_object = shift;
+
+    my ($msg_entity, $msg_hash);
+
+    if (ref($msg_object) =~ /^MIME::Entity/) { ## MIME-ttols object
+	$msg_entity = $msg_object;
+    }elsif (ref($msg_object) =~ /^Message/) { ## Sympa's own Message object
+	$msg_entity = $msg_object->{'msg'};
+    }else {
+	&do_log('err', "reject_report_msg: wrong type for msg parameter");
+    }
+    
+    my $head = $msg_entity->head;
+    my $body_handle = $msg_entity->bodyhandle;
+    my $body_as_string;
+    
+    if (defined $body_handle) {
+	$body_as_string = $body_handle->as_lines();
+    }
+
+    ## TODO : we should also decode headers + remove trailing \n + use these variables in default mail templates
+
+    $msg_hash = {'full' => $msg_entity->as_string, 
+		 'body' => $body_as_string,
+		 'from' => $head->get('From'),
+		 'subject' => $head->get('Subject'),
+		 'message_id' => $head->get('Message-Id')
+		 };
+
+    return $msg_hash;
+}
+
 ############################################################
 #  notice_report_msg
 ############################################################
@@ -154,6 +206,11 @@ sub notice_report_msg {
 	return undef;
     }
 
+    ## Prepare the original message if provided
+    if (defined $param->{'message'}) {
+	$param->{'original_msg'} = &_get_msg_as_hash($param->{'message'});
+     }
+
     if (ref($list) eq "List") {
 	unless ($list->send_file('message_report',$user,$robot,$param)) {
 	    &do_log('notice',"report::notice_report_msg(): Unable to send template 'message_report' to '$user'");
