@@ -1,7 +1,7 @@
 #!--PERL--
 
 # archived.pl - This script does the web archives building for Sympa
-# RCS Identication ; $Revision: 1.68.2.2 $ ; $Date: 2006/08/29 11:25:36 $ 
+# RCS Identication ; $Revision: 4471 $ ; $Date: 2007-06-26 15:35:43 +0200 (mar, 26 jun 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -25,6 +25,18 @@
 ##         :  d		-> debug -d is equiv to -dF
 ## Now, it is impossible to use -dF but you have to write it -d -F
 
+=pod 
+
+=head1 NAME 
+
+I<archived.pl> - Daemon running the web archive building.
+
+=head1 DESCRIPTION 
+
+This script must be run along with sympa. It regularly checks the 'outgoing' spool and picks the messages it finds in it. It then calls MHonArc to build the HTML version of archives and add an SMTP text version (i.e. an ASCII file including headers and body) to the appropriate directory.
+
+=cut 
+
 ## Change this to point to your Sympa bin directory
 use lib '--LIBDIR--';
 
@@ -44,6 +56,9 @@ use Version;
 require 'tt2.pl';
 require 'tools.pl';
 
+my $daemon_name = &Log::set_daemon($0);
+my $ip = $ENV{'REMOTE_HOST'};
+
 #getopts('dF');
 
 ## Check options
@@ -55,6 +70,7 @@ if ($main::options{'debug'}) {
 }
 
 $main::options{'foreground'} = 1 if ($main::options{'debug'});
+$main::options{'log_to_stderr'} = 1 if ($main::options{'debug'} || $main::options{'foreground'});
 
 $wwsympa_conf = "--WWSCONFIG--";
 $sympa_conf_file = '--CONFIG--';
@@ -87,11 +103,6 @@ unless ($List::use_db = &List::check_db_connect()) {
     &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable. If you don\'t use any database, comment db_xxx parameters in sympa.conf', $Conf{'db_name'});
 }
 
-## Check databse connectivity
-unless ($List::use_db = &List::check_db_connect()) {
-    &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable. If you don\'t use any database, comment db_xxx parameters in sympa.conf', $Conf{'db_name'});
-}
-
 ## Put ourselves in background if not in debug mode. 
 unless ($main::options{'debug'} || $main::options{'foreground'}) {
    open(STDERR, ">> /dev/null");
@@ -125,6 +136,11 @@ $< = $> = (getpwnam('--USER--'))[2];
 &POSIX::setuid((getpwnam('--USER--'))[2]);
 &POSIX::setgid((getgrnam('--GROUP--'))[2]);
 
+## Check if the UID has correctly been set (usefull on OS X)
+unless (($( == (getgrnam('--GROUP--'))[2]) && ($< == (getpwnam('--USER--'))[2])) {
+    &fatal_err("Failed to change process userID and groupID. Note that on some OS Perl scripts can't change their real UID. In such circumstances Sympa should be run via SUDO.");
+}
+
 ## Sets the UMASK
 umask(oct($Conf{'umask'}));
 
@@ -135,7 +151,6 @@ unless ((-r $wwsconf->{'arc_path'}) && (-w $wwsconf->{'arc_path'})) {
 
 ## Change to list root
 unless (chdir($Conf{'home'})) {
-    &report::reject_report_web('intern','chdir_error',{},'','','',$robot);
     &do_log('err','unable to change directory');
     exit (-1);
 }
@@ -200,8 +215,11 @@ while (!$end) {
 	        do_log ('err',"Ignoring file $queue/$file because couldn't read it, archived.pl must use the same uid as sympa");
 		   next;
 	       }
+
+	   my $email_regexp = &tools::get_regexp('email');
+
 	   foreach my $removeorder (<REMOVE>) { 
-	       unless($removeorder =~ /(.*)\|\|($tools::regexp{'email'})/){
+	       unless($removeorder =~ /(.*)\|\|($email_regexp)/){
 		   do_log ('err',"Ignoring remove_order $removeorder not recognized format");   
 		   next;
 	       }
@@ -327,17 +345,93 @@ while (!$end) {
    }
 }
 do_log('notice', 'archived exited normally due to signal');
-unlink("$wwsconf->{'archived_pidfile'}");
+&tools::remove_pid($wwsconf->{'archived_pidfile'}, $$);
 
 exit(0);
 
 
+=pod 
+
+=head1 SUBFUNCTIONS 
+
+This is the description of the subfunctions contained by archived.pl.
+
+=cut 
+
+=pod 
+
+=head2 sub sigterm()
+
+Switches the loop control variable $end value to 1 when SIGTERM signal is caught.
+
+=head3 Arguments 
+
+=over 
+
+=item * I<none> 
+
+=back 
+
+=head3 Return 
+
+=over 
+
+=item * I<none> 
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * none
+
+=back 
+
+=cut 
+
 ## When we catch SIGTERM, just change the value of the loop
 ## variable.
 sub sigterm {
     $end = 1;
 }
 
+=pod 
+
+=head2 sub remove(STRING $adrlist, STRING $msgid)
+
+Removes the message having the identifier $msgid from the list named $adrlist.
+
+=head3 Arguments 
+
+=over 
+
+=item * I<$adrlist>, a character string containing the list name.
+
+=item * I<$msgid> , a character string containing the message identifier.
+
+=back 
+
+=head3 Return 
+
+=over 
+
+=item * I<none> 
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * Log::db_log
+
+=item * Log::do_log
+
+=back 
+
+=cut 
+
 sub remove {
     my $adrlist = shift;
     my $msgid = shift;
@@ -351,6 +445,9 @@ sub remove {
     }
 
     do_log('notice',"Removing $msgid in list $adrlist section $2");
+    unless(&Log::db_log({'robot' => $robot,'list' => $adrlist,'action' => 'remove','parameters' => $msgid.','.$adrlist,'target_email' => '','msg_id' => $msgid,'status' => 'succes','error_type' => '','user_email' =>'','client' => $ip,'daemon' => $daemon_name})) {
+	&do_log('error','archived::remove: unable to log event');
+    }
   
     $arc =~ /^(\d{4})-(\d{2})$/ ;
     my $yyyy = $1 ;
@@ -360,6 +457,50 @@ sub remove {
     system "$wwsconf->{'mhonarc'}  -outdir $wwsconf->{'arc_path'}/$adrlist/$yyyy-$mm -rmm $msgid";
 }
 
+=pod 
+
+=head2 sub rebuild(STRING $adrlist)
+
+Rebuilds archives for the list the name of which is given in the argument $adrlist.
+
+=head3 Arguments 
+
+=over 
+
+=item * I<$adrlist>, a character string containing the name of the list the archives of which we want to rebuild.
+
+=back 
+
+=head3 Return 
+
+=over 
+
+=item * I<undef> if something goes wrong.
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * get_tag
+
+=item * set_hidden_mode
+
+=item * unset_hidden_mode
+
+=item * List::new
+
+=item * Log::do_log
+
+=item * tools::get_filename
+
+=item * tools::remove_dir
+
+=back 
+
+=cut 
+
 sub rebuild {
 
     my $adrlist = shift;
@@ -387,7 +528,7 @@ sub rebuild {
 
     do_log('debug',"Rebuilding $adrlist archive ($2)");
 
-    my $mhonarc_ressources = &tools::get_filename('etc','mhonarc-ressources.tt2',$list->{'domain'}, $list);
+    my $mhonarc_ressources = &tools::get_filename('etc',{},'mhonarc-ressources.tt2',$list->{'domain'}, $list);
 
     if (($list->{'admin'}{'web_archive_spam_protection'} ne 'none') && ($list->{'admin'}{'web_archive_spam_protection'} ne 'cookie')) {
 	&set_hidden_mode($tag);
@@ -493,6 +634,74 @@ sub rebuild {
 }
 
 
+=pod 
+
+=head2 sub mail2arc(STRING $file,STRING $listname,STRING $hostname,STRING $yyyy,STRING $mm,STRING $dd,STRING $hh,STRING $min,STRING $ss)
+
+Archives one message into one list archives directory.
+
+=head3 Arguments 
+
+=over 
+
+=item * I<$file>: a character string containing the message filename.
+
+=item * I<$listname>: a character string containing the name of the list in which to archive the message
+
+=item * I<$hostname>: a character string containing the name of the virtual robot hosting the list.
+
+=item * I<$yyyy>: a character string containing the year of the date when the message is archived (i.e. now)
+
+=item * I<$mm>: a character string containing the month of the date when the message is archived (i.e. now)
+
+=item * I<$dd>: a character string containing the day of the date when the message is archived (i.e. now)
+
+=item * I<$hh>: a character string containing the hour of the date when the message is archived (i.e. now)
+
+=item * I<$min>: a character string containing the minute of the date when the message is archived (i.e. now)
+
+=item * I<$ss>: a character string containing the second of the date when the message is archived (i.e. now)
+
+=back 
+
+=head3 Return 
+
+=over 
+
+=item * I<undef> if something goes wrong.
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * get_tag
+
+=item * save_idx
+
+=item * set_hidden_mode
+
+=item * unset_hidden_mode
+
+=item * List::get_arc_size
+
+=item * List::get_list_id
+
+=item * List::new
+
+=item * List::send_notify_to_owner
+
+=item * Log::do_log
+
+=item * tools::get_filename
+
+=item * tools::remove_dir
+
+=back 
+
+=cut 
+
 sub mail2arc {
 
     my ($file, $listname, $hostname, $yyyy, $mm, $dd, $hh, $min, $ss) = @_;
@@ -501,6 +710,11 @@ sub mail2arc {
 
     my $list = new List($listname, $hostname);
 
+    unless (defined $list) {
+	&do_log('err', 'Unknown list %s@%s', $listname, $hostname);
+	return undef;
+    }
+
     my $tag = &get_tag($listname);
 
     if (($list->{'admin'}{'web_archive_spam_protection'} ne 'none') && ($list->{'admin'}{'web_archive_spam_protection'} ne 'cookie')) {
@@ -596,7 +810,7 @@ sub mail2arc {
 	$newfile = $index;
      }
     
-    my $mhonarc_ressources = &tools::get_filename('etc','mhonarc-ressources.tt2',$list->{'domain'}, $list);
+    my $mhonarc_ressources = &tools::get_filename('etc',{},'mhonarc-ressources.tt2',$list->{'domain'}, $list);
     
     do_log ('debug',"calling $wwsconf->{'mhonarc'} for list %s", $list->get_list_id() ) ;
     my $cmd = "$wwsconf->{'mhonarc'} -add -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\'  -rcfile $mhonarc_ressources -outdir $monthdir  -definevars \"listname='$listname' hostname=$hostname yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$wwsconf->{'arc_path'} base=$Conf{'wwsympa_url'}/arc tag=$tag\" -umask $Conf{'umask'} < $queue/$file";
@@ -630,6 +844,38 @@ sub mail2arc {
     &save_idx("$monthdir/index",$newfile);
 }
 
+=pod 
+
+=head2 sub set_hidden_mode(STRING $tag)
+
+Sets the value of $ENV{'M2H_ADDRESSMODIFYCODE'} and $ENV{'M2H_MODIFYBODYADDRESSES'}
+
+=head3 Arguments 
+
+=over 
+
+=item * I<$tag> a character string (containing the result of get_tag($listname))
+
+=back 
+
+=head3 Return 
+
+=over 
+
+=item * I<none> 
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * none
+
+=back 
+
+=cut 
+
 sub set_hidden_mode {
     my $tag = shift; ## tag is used as variable elements in tags to prevent message contents to be parsed
 
@@ -638,12 +884,78 @@ sub set_hidden_mode {
     $ENV{'M2H_MODIFYBODYADDRESSES'} = 1;
 }
 
+=pod 
+
+=head2 sub unset_hidden_mode()
+
+Empties $ENV{'M2H_ADDRESSMODIFYCODE'}.
+
+=head3 Arguments 
+
+=over 
+
+=item * I<none> 
+
+=back 
+
+=head3 Return 
+
+=over 
+
+=item * I<none> 
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * none
+
+=back 
+
+=cut 
+
 sub unset_hidden_mode {
     
     ## Be carefull, the .mhonarc.db file keeps track of previous M2H_ADDRESSMODIFYCODE setup
     $ENV{'M2H_ADDRESSMODIFYCODE'} = '';
 }
 
+=pod 
+
+=head2 sub save_idx(STRING $index,STRING $lst)
+
+Saves the archives index file
+
+=head3 Arguments 
+
+=over 
+
+=item * I<$index>, a string corresponding to the file name to which save an index.
+
+=item * I<$lst>, a character string
+
+=back 
+
+=head3 Return 
+
+=over 
+
+=item * I<none> 
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * none
+
+=back 
+
+=cut 
+
 sub save_idx {
     my ($index,$lst) = @_;
     
@@ -653,8 +965,62 @@ sub save_idx {
     #   do_log('debug',"last arc entry for $index is $lst");
 }
 
+=pod 
+
+=head2 sub get_tag(STRING $listname)
+
+Returns a tag derived from the listname.
+
+=head3 Arguments 
+
+=over 
+
+=item * I<$listname>, a character string correspondiong to the list name.
+
+=back 
+
+=head3 Return 
+
+=over 
+
+=item * I<a character string>, corresponding to the 10 last characters of a 32 bytes string containing the MD5 digest of the concatenation of the following strings (in this order):
+
+=over 4
+
+=item - the cookie config parameter
+
+=item - a slash: "/"
+
+=item - the I<$listname> argument
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * Digest::MD5::md5_hex
+
+=back 
+
+=cut 
+
 sub get_tag {
     my $listname = shift;
     
     return (substr(Digest::MD5::md5_hex(join('/', $Conf{'cookie'}, $listname)), -10)) ;
 }
+
+=pod 
+
+=head1 AUTHORS 
+
+=over 
+
+=item * Serge Aumont <sa AT cru.fr> 
+
+=item * Olivier Salaun <os AT cru.fr> 
+
+=back 
+
+=cut 
