@@ -1,5 +1,5 @@
 # bounce-lib.pl - This module includes functions for analysing non-delivery reports
-# RCS Identication ; $Revision: 1.8.2.1 $ ; $Date: 2006/05/02 14:13:16 $ 
+# RCS Identication ; $Revision: 4543 $ ; $Date: 2007-08-17 11:40:59 +0200 (ven, 17 aoû 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -72,7 +72,7 @@ sub rfc1891 {
 	    while (<BODY>) {
 
 		my ($status, $recipient);
-		if (/^Status:\s*(\d+\.\d+\.\d+)$/mi) {
+		if (/^Status:\s*(\d+\.\d+\.\d+)(\s|$)/mi) {
 		    $status = $1;
 		}
 		
