@@ -1,5 +1,5 @@
 # wwslib.pm - This module includes functions used by wwsympa.fcgi
-# RCS Identication ; $Revision: 1.53.2.1 $ ; $Date: 2006/12/21 09:09:20 $ 
+# RCS Identication ; $Revision: 4455 $ ; $Date: 2007-06-19 15:40:16 +0200 (mar, 19 jun 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -40,7 +40,7 @@ use Conf;
 		   'txt' => {'gettext_id' => 'text only mode'},
 		   'html'=> {'gettext_id' => 'html only mode'},
 		   'urlize' => {'gettext_id' => 'urlize mode'},
-		   'nomail' => {'gettext_id' => 'no mail (usefull for vacations)'},
+		   'nomail' => {'gettext_id' => 'no mail (useful for vacations)'},
 		   'not_me' => {'gettext_id' => 'you do not receive your own posts'}
 		   );
 
@@ -59,7 +59,7 @@ use Conf;
 
 ## Filenames with corresponding entry in NLS set 15
 %filenames = ('welcome.tt2'             => {'gettext_id' => "welcome message"},
-	      'bye.tt2'                 => {'gettext_id' => "unsubscription message"},
+	      'bye.tt2'                 => {'gettext_id' => "unsubscribe message"},
 	      'removed.tt2'             => {'gettext_id' => "deletion message"},
 	      'message.footer'          => {'gettext_id' => "message footer"},
 	      'message.header'          => {'gettext_id' => "message header"},
@@ -151,6 +151,7 @@ sub load_config {
 			icons_url => '/icons',
 			mhonarc => '/usr/bin/mhonarc',
 			review_page_size => 25,
+			viewlogs_page_size => 25,
 			task_manager_pidfile => '--PIDDIR--/task_manager.pid',
 			title => 'Mailing Lists Service',
 			use_fast_cgi => 1,
@@ -254,20 +255,24 @@ sub load_mime_types {
 ## Returns user information extracted from the cookie
 sub get_email_from_cookie {
 #    &Log::do_log('debug', 'get_email_from_cookie');
+    my $cookie = shift;
     my $secret = shift;
+
     my ($email, $auth) ;
 
-    unless ($secret) {
+    # &Log::do_log('info', "get_email_from_cookie($cookie,$secret)");
+    
+    unless (defined $secret) {
 	&report::reject_report_web('intern','cookie_error',{},'','','',$robot);
-	&Log::do_log('info', 'parameter cookie undefine, authentication failure');
+	&Log::do_log('info', 'parameter cookie undefined, authentication failure');
     }
 
-    unless ($ENV{'HTTP_COOKIE'}) {
-	&report::reject_report_web('intern','cookie_error_env',{'env'=> ENV{HTTP_COOKIE}},'get_email_from_cookie','','',$robot);
-	&Log::do_log('info', ' ENV{HTTP_COOKIE} undefined, authentication failure');
+    unless ($cookie) {
+	&report::reject_report_web('intern','cookie_error',$cookie,'get_email_from_cookie','','',$robot);
+	&Log::do_log('info', ' cookie undefined, authentication failure');
     }
 
-    ($email, $auth) = &cookielib::check_cookie ($ENV{'HTTP_COOKIE'}, $secret);
+    ($email, $auth) = &cookielib::check_cookie ($cookie, $secret);
     unless ( $email) {
 	&report::reject_report_web('user','auth_failed',{},'');
 	&Log::do_log('info', 'get_email_from_cookie: auth failed for user %s', $email);
@@ -295,45 +300,6 @@ sub valid_email {
     $email =~ /^([\w\-\_\.\/\+\=]+|\".*\")\@[\w\-]+(\.[\w\-]+)+$/;
 }
 
-# create a cipher
-sub ciphersaber_installed {
-    if (eval "require Crypt::CipherSaber") {
-	require Crypt::CipherSaber;
-	return &Crypt::CipherSaber->new($Conf{'cookie'});
-    }else{
-	return ('no_cipher');
-    }
-}
-
-## encrypt a password
-sub crypt_passwd {
-    my $inpasswd = shift ;
-
-    unless (defined($cipher)){
-	$cipher = ciphersaber_installed();
-    }
-    return $inpasswd if ($cipher eq 'no_cipher') ;
-    return ("crypt.".$cipher->encrypt ($inpasswd)) ;
-}
-
-## decrypt a password
-sub decrypt_passwd {
-    my $inpasswd = shift ;
-
-    return $inpasswd unless ($inpasswd =~ /^crypt\.(.*)$/) ;
-    $inpasswd = $1;
-
-    unless (defined($cipher)){
-	$cipher = ciphersaber_installed();
-    }
-    if ($cipher eq 'no_cipher') {
-	do_log('info','password seems crypted while CipherSaber is not installed !');
-	return $inpasswd ;
-    }
-    return $cipher->decrypt ($inpasswd);
-}
-
-
 sub init_passwd {
     my ($email, $data) = @_;
     
@@ -376,7 +342,7 @@ sub get_my_url {
     my $return_url;
     
     ## Mod_ssl sets SSL_PROTOCOL ; apache-ssl sets SSL_PROTOCOL_VERSION
-    if ($ENV{SSL_PROTOCOL} || $ENV{SSL_PROTOCOL_VERSION}) {
+    if ($ENV{'HTTPS'} eq 'on') {
 	$return_url = 'https';
     }else{
 	$return_url = 'http';	
